/*******************************************************************************
 *
 * Copyright (c) 2016-2026 HYCON Technology, Inc.
 * All rights reserved.
 * HYCON Technology <www.hycontek.com>
 *
 * HYCON reserves the right to amend this code without notice at any time.
 * HYCON assumes no responsibility for any errors appeared in the code,
 * and HYCON disclaims any express or implied warranty, relating to sale
 * and/or use of this code including liability or warranties relating
 * to fitness for a particular purpose, or infringement of any patent,
 * copyright or other intellectual property right.
 *
 * -----------------------------------------------------------------------------
 * Project Name : HY16F188_CMP
 * IDE tooling  : AndeSight C/C++ IDE, version: 2.1.1 Build ID : 201608241332
 * Device Ver.  : HY16F_RDSp3_DeviceV0.2 crt0.o for HY16F18x MCU.
 * Library Ver. : 1.5
 * MCU Device   : 
 * Description  : 
 * Created Date : 2018/2/18
 * Created by   :
 *
 * Program Description:
 * --------------------
 *
 * ---------
 * HY16F188 |              --------------------
 *    PT2.0 | SCL ---> SCL | LCD Drive HY2613 |
 *    PT2.1 | SDA ---> SDA --------------------
 *     GND  |
 *          |
 * ---------
 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include "HY16F188.h"
#include "System.h"
#include "DrvGPIO.h"
#include "DrvCLOCK.h"
#include "DrvPMU.h"
#include "DrvREG32.h"
#include "HY2613.h"
#include "DrvI2C.h"
#include "DrvCMP.h"
#include "my define.h"

/*----------------------------------------------------------------------------*/
/* STRUCTURES                                                                 */
/*----------------------------------------------------------------------------*/
volatile typedef union _MCUSTATUS
{
  char  _byte;
  struct
  {
    unsigned b_ADCdone:1;
    unsigned b_TMAdone:1;
    unsigned b_TMBdone:1;
    unsigned b_TMC0done:1;
    unsigned b_TMC1done:1;
    unsigned b_CMPdone:1;
    unsigned b_UART_TxDone:1;
    unsigned b_UART_RxDone:1;
  };
} MCUSTATUS;


/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#define  I2CBufferSize  64
#define I2C_WRITE       0x00           // I2C WRITE command
#define I2C_READ        0x01           // I2C READ command
#define CMPO_PORT E_PT1
#define CMPOut BIT7

//#define V12_Voltage_mueasure
//#define RLO_Voltage_mueasure
#define CH3_Compare_CH2_Voltage
/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
unsigned char I2C_RW;
unsigned char I2C_TARGET;
unsigned char I2C_EndFlag;
unsigned int I2C_Sendbuf[I2CBufferSize];
unsigned int I2C_Recbuf[I2CBufferSize];
unsigned int I2C_DataTxLen,I2C_DataTxIndex,I2C_DataRxLen,I2C_DataRxIndex;

MCUSTATUS  MCUSTATUSbits;

/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);
void InitalI2C(void);

/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
int main(void)
{

  InitalI2C();
  SYS_EnableGIE(4,0x3F);       //Enable GIE(Global Interrupt)
  MCUSTATUSbits._byte = 0;

  //Display Setting
  DisplayInit();
  ClearLCDframe();
  DisplayHYcon();

  //GPIO Setting & GPIO Setting
  DrvGPIO_Open(CMPO_PORT,CMPOut,E_IO_OUTPUT);  //Set PT1_7 OUTPUT

#if defined(RLO_Voltage_mueasure)
  DrvCMP_PInput(0);            //CMP positive input CH1
  DrvCMP_NInput(3);            //CMP negative input RLO
  DrvCMP_RLO_refV(2,1);        //RLO=VDD3V, CPIS=Closed
  DrvCMP_InputSwitch(1);       //INPUT SHORT SWITCH ENABLE
#endif

#if defined(V12_Voltage_mueasure)
  DrvPMU_VDDA_Voltage(E_VDDA2_4);
  DrvPMU_VDDA_LDO_Ctrl(E_LDO);
  DrvPMU_BandgapEnable();
  DrvPMU_REFO_Enable();
  DrvCMP_PInput(3);            //CMP positive input V12
  DrvCMP_NInput(0);            //CMP negative input CH1
  DrvCMP_InputSwitch(1);       //INPUT SHORT SWITCH ENABLE
#endif

#if defined(CH3_Compare_CH2_Voltage)
  unsigned int i=0;
  DrvCMP_PInput(2);            //CMP positive input CH3
  DrvCMP_NInput(1);            //CMP negative input CH2
  DrvCMP_InputSwitch(0);       //INPUT SHORT SWITCH ENABLE
#endif

  DrvCMP_Enable();             //CMP enable.
  DrvCMP_OutputPinEnable(0);   //Enable CMP digital output to port
	                           //0 : PT1.7

#if defined(RLO_Voltage_mueasure)
  // Setting CPDM=0000b and CPDA to observe RLO voltage
  DrvCMP_RLO_Ctrl(0x00,0x00);  //CH1 to VSS around 0.0062V
  DrvCMP_RLO_Ctrl(0x01,0x00);  //CH1 to VSS around 0.2132V
  DrvCMP_RLO_Ctrl(0x02,0x00);  //CH1 to VSS around 0.4163V
  DrvCMP_RLO_Ctrl(0x03,0x00);  //CH1 to VSS around 0.6205V
  DrvCMP_RLO_Ctrl(0x04,0x00);  //CH1 to VSS around 0.8221V
  DrvCMP_RLO_Ctrl(0x05,0x00);  //CH1 to VSS around 1.0252V
  DrvCMP_RLO_Ctrl(0x06,0x00);  //CH1 to VSS around 1.2290V
  DrvCMP_RLO_Ctrl(0x07,0x00);  //CH1 to VSS around 1.4355V
  DrvCMP_RLO_Ctrl(0x08,0x00);  //CH1 to VSS around 1.6393V
  DrvCMP_RLO_Ctrl(0x09,0x00);  //CH1 to VSS around 1.8449V
  DrvCMP_RLO_Ctrl(0x0A,0x00);  //CH1 to VSS around 2.0476V
  DrvCMP_RLO_Ctrl(0x0B,0x00);  //CH1 to VSS around 2.2535V
  DrvCMP_RLO_Ctrl(0x0C,0x00);  //CH1 to VSS around 2.4577V
  DrvCMP_RLO_Ctrl(0x0D,0x00);  //CH1 to VSS around 2.6613V
  DrvCMP_RLO_Ctrl(0x0E,0x00);  //CH1 to VSS around 2.8686V
  DrvCMP_RLO_Ctrl(0x0F,0x00);  //CH1 to VSS around 3.0777V, VDD3V around 3.3091V
#endif

#if defined(CH3_Compare_CH2_Voltage)
  while(1)
  {
    i=DrvCMP_ReadData();
    LCD_DATA_DISPLAY(i);       //IF positive(CH3) > negative(CH2), CMPO=High, LCD Display(1). otherwise CMPO=Low, LCD Display(0).
  }
#endif

  while(1);
  return 0;
}

/*----------------------------------------------------------------------------*/
/* Function Name: HW0_ISR()                                                   */
/* Description  : I2C/UART/SPI interrupt Service Routine (HW0).               */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW0_ISR(void)
{
  unsigned char I2C_Status,I2C_IntFlag;

  I2C_IntFlag=DrvI2C_ReadIntFlag();
  if((I2C_IntFlag == E_DRVI2C_INT)||(I2C_IntFlag == E_DRVI2C_INT_ALL))  //Get I2C Interrupt Flag
  {
    I2C_Status=DrvI2C_GetStatusFlag();     //Get I2C Status Flag
    switch(I2C_Status)
    {
      case 0x90:  //MACTFlag+RWFlag
          {       //START has been transmitted
            DrvI2C_WriteData(I2C_TARGET);  //Send Slave Address & R/W Bit
            DrvI2C_Ctrl(0,0,0,0);          //Clear all I2C flag
            break;
          };
      case 0x84:  //MACTFlag+ACKFlag
          {       //Slave A + W has been transmitted. ACK has been received.
            DrvI2C_WriteData(I2C_Sendbuf[I2C_DataTxIndex++]);  //Send Data to Slave
            DrvI2C_Ctrl(0,0,0,0);      //Clear all I2C flag
            break;
          };
      case 0x80: //MACTFlag
          {      //Slave A + W has been transmitted. ACK has been received.
            DrvI2C_WriteData(I2C_Sendbuf[I2C_DataTxIndex++]);  //Send Data to Slave
            DrvI2C_Ctrl(0,0,0,0);      //Clear all I2C flag
            break;
          };
      case 0x30:
          {
            DrvI2C_Ctrl(0,0,0,0);      //Clear all I2C flag
            I2C_EndFlag=1;
            break;
          };
      case 0x8C:  //MACTFlag+DFFlag+ACKFlag
          {       //DATA has been transmitted and ACK has been received
            if(I2C_DataTxIndex<I2C_DataTxLen)
            {
              DrvI2C_WriteData(I2C_Sendbuf[I2C_DataTxIndex++]);  //Send Data to Slave
              DrvI2C_Ctrl(0,0,0,0);    // Clear all I2C flag
            }
            else
            {
              if(I2C_RW == I2C_WRITE)
              {
                DrvI2C_Ctrl(0,1,0,0);  //I2C as master sends STOP signal
                I2C_EndFlag=1;
              }
              else if(I2C_RW == I2C_READ)
                DrvI2C_Ctrl(1,0,0,0);  //I2C as master sends START signal
              I2C_DataTxIndex=0;
            }
            break;
          };
      case 0x88:  //MACTFlag+DFFlag
          {       //DATA has been transmitted and NACK has been received
            DrvI2C_Ctrl(0,1,0,0);    //I2C as master sends STOP signal
            I2C_DataTxIndex=0;
            I2C_EndFlag=1;
            break;
          };
      case 0xB0:
          {       //A repeated START has been transmitted.
            DrvI2C_WriteData(I2C_TARGET | I2C_READ);  //Send Slave Address & R/W Bit
            DrvI2C_Ctrl(0,0,0,0);    //Clear all I2C flag
            break;
          }
      case 0x94:  //MACTFlag+RWFlag+ACKFlag
          {       //Slave A + R has been transmitted. ACK has been received.
            if(I2C_DataRxLen>1)
            {
              DrvI2C_Ctrl(0,0,0,1);  //Set ACK bit
            }else{
              DrvI2C_Ctrl(0,0,0,0);  //Clear all I2C flag
            }
            break;
          };
      case 0x9C:  //MACTFlag+RWFlag+DFFlag+ACKFlag
          {       //Data byte has been received. ACK has been transmitted.
            I2C_Recbuf[I2C_DataRxIndex++]=DrvI2C_ReadData();
            if((I2C_DataRxLen-1)>I2C_DataRxIndex)
            {
              DrvI2C_Ctrl(0,0,0,1);  //Set ACK bit
            }
            else
            {
              DrvI2C_Ctrl(0,0,0,0);  //Clear all I2C flag
            }
            break;
          };
      case 0x98:  //MACTFlag+RWFlag+DFFlag
          {       //Data byte has been received. NACK has been transmitted.
            I2C_Recbuf[I2C_DataRxIndex++]=DrvI2C_ReadData();
            DrvI2C_Ctrl(0,1,0,0);  //I2C as master sends STOP signal
            I2C_EndFlag=1;
            break;
          };
      default:
          {
            DrvI2C_Ctrl(0,0,0,0);  //Clear all I2C flag
            I2C_EndFlag=1;
            break;
          };
    }
    DrvI2C_ClearIRQ();
    DrvI2C_ClearEIRQ();            //Clear EIRQFlag
    DrvI2C_ClearIntFlag(0);        //Clear I2C Interrupt Flag(I2CIF)
  }
  if((I2C_IntFlag == E_DRVI2C_ERROR_INT)||(I2C_IntFlag == E_DRVI2C_INT_ALL))  //Get I2C Error Interrupt Flag
  {
    I2C_EndFlag=1;
    DrvI2C_ClearIRQ();
    DrvI2C_ClearEIRQ();            //Clear EIRQFlag
    DrvI2C_ClearIntFlag(1);        //Clear I2C Interrupt Flag(I2CEIF)
    DrvI2C_Ctrl(0,0,0,0);          //Clear all I2C flag
  }
  SYS_EnableGIE(4,0x3F);           //Enable GIE(Global Interrupt)

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW1_ISR()                                                   */
/* Description  : WDT & RTC & Timer A/B/C interrupt Service Routine (HW1).    */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW1_ISR(void)
{

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW2_ISR()                                                   */
/* Description  : ADC interrupt Service Routine (HW2).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW2_ISR(void)
{

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW3_ISR()                                                   */
/* Description  : CMP & OPA interrupt Service Routine (HW3).                  */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW3_ISR(void)
{
  MCUSTATUSbits.b_CMPdone=1;
  DrvCMP_ClearIntFlag();       //Clear CMP interrupt flag
}

/*----------------------------------------------------------------------------*/
/* Function Name: HW4_ISR()                                                   */
/* Description  : PT1 interrupt Service Routine (HW4).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW4_ISR(void)
{

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW5_ISR()                                                   */
/* Description  : PT2 interrupt Service Routine (HW5).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW5_ISR(void)
{

}

/*----------------------------------------------------------------------------*/
/* Function Name: tlb_exception_handler()                                     */
/* Description  : Exception Service Routines.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void tlb_exception_handler()
{
  asm("nop"); //procedure define by customer.
  asm("nop");
}

/*----------------------------------------------------------------------------*/
/* Software Delay Subroutines                                                 */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
  for(;num>0;num--)
  asm("NOP");
}

/*----------------------------------------------------------------------------*/
/* Function Name: InitalI2C(void)                                             */
/* Description  : Hardware I2C Initial.                                       */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void InitalI2C(void)
{
  DrvI2C_SetIOPin(4);    //setting io pin, 4 SCL=PT2.0;SDA=PT2.1
  DrvI2C_Open(0x4);      //Enable I2C function and set I2C baud rate=100kHz
  //Default CPU clock is 2MHz, Data Baud Rate : (I2CLK)/[4x(CRG+1)]= 2000000/[4*(4+1)]=100kHz
  DrvI2C_EnableInt(2);   //Enable I2C interrupt and error interrupt
}

/*---------------------------------------------------------------------------*/
/* End Of File                                                               */
/*---------------------------------------------------------------------------*/
