/*******************************************************************************
 *
 * Copyright (c) 2016-2026 HYCON Technology, Inc.
 * All rights reserved.
 * HYCON Technology <www.hycontek.com>
 *
 * HYCON reserves the right to amend this code without notice at any time.
 * HYCON assumes no responsibility for any errors appeared in the code,
 * and HYCON disclaims any express or implied warranty, relating to sale
 * and/or use of this code including liability or warranties relating
 * to fitness for a particular purpose, or infringement of any patent,
 * copyright or other intellectual property right.
 *
 * -----------------------------------------------------------------------------
 * Project Name : HY16F188_GPIO
 * IDE tooling  : AndeSight C/C++ IDE, version: 2.1.1 Build ID : 201608241332
 * Device Ver.  : HY16F_RDSp3_DeviceV0.2 crt0.o for HY16F18x MCU.
 * Library Ver. : 1.5
 * MCU Device   : 
 * Description  : 
 * Created Date : 2018/2/18
 * Created by   :
 *
 * Program Description:
 * --------------------
 *
 * ---------
 * HY16F188 |              --------------------
 *    PT2.0 | SCL ---> SCL | LCD Drive HY2613 |
 *    PT2.1 | SDA ---> SDA --------------------
 *     GND  |
 *          |
 * ---------
 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include "HY16F188.h"
#include "System.h"
#include "DrvI2C.h"
#include "DrvCLOCK.h"
#include "HY2613.h"
#include "my define.h"
#include "DrvGPIO.h"

/*----------------------------------------------------------------------------*/
/* STRUCTURES                                                                 */
/*----------------------------------------------------------------------------*/
volatile typedef union _MCUSTATUS
{
  char  _byte;
  struct
  {
    unsigned b_ADCdone:1;
    unsigned b_TMAdone:1;
    unsigned b_TMBdone:1;
    unsigned b_TMC0done:1;
    unsigned b_WDTdone:1;
    unsigned b_RTCdone:1;
    unsigned b_UART_TxDone:1;
    unsigned b_UART_RxDone:1;
  };
} MCUSTATUS;

typedef union _PTINTSTATUS
{
  char  _byte;
  struct
  {
    unsigned b_PTINT0done:1;
    unsigned b_PTINT1done:1;
    unsigned b_PTINT2done:1;
    unsigned b_PTINT3done:1;
    unsigned b_PTINT4done:1;
    unsigned b_PTINT5done:1;
    unsigned b_PTINT6Done:1;
    unsigned b_PTINT7Done:1;
  };
} PTINTSTATUS;


/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#define  I2CBufferSize  64
#define I2C_WRITE       0x00           // I2C WRITE command
#define I2C_READ        0x01           // I2C READ command
#define KEY_PORT E_PT1
#define KEYIN0 BIT6
#define KEYIN1 BIT7
#define KEYIN0_PIN 6
#define KEYIN1_PIN 7

/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
unsigned char I2C_RW;
unsigned char I2C_TARGET;
unsigned char I2C_EndFlag;
unsigned int I2C_Sendbuf[I2CBufferSize];
unsigned int I2C_Recbuf[I2CBufferSize];
unsigned int I2C_DataTxLen,I2C_DataTxIndex,I2C_DataRxLen,I2C_DataRxIndex;

MCUSTATUS  MCUSTATUSbits;
PTINTSTATUS  PT1INTSTATUSbits;
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);
void InitalI2C(void);

/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
int main(void)
{

  int count=0;

  DrvGPIO_ClkGenerator(E_HS_CK,1);                      //Set IO sampling clock input source is HS_CK
  DrvGPIO_Open(KEY_PORT,KEYIN1|KEYIN0,E_IO_INPUT);      //set PT1.2/PT1.1 INPUT
  DrvGPIO_Open(KEY_PORT,KEYIN1|KEYIN0,E_IO_PullHigh);   //enable PT1.2/PT1.1 pull high R
  DrvGPIO_Open(KEY_PORT,KEYIN1|KEYIN0,E_IO_IntEnable);  //PT1.2/PT1.1 interrupt enable
  DrvGPIO_IntTrigger(KEY_PORT,KEYIN1|KEYIN0,E_N_Edge);  //PT1.2/PT1.1 interrupt trigger method is negative edge
  DrvGPIO_ClearIntFlag(KEY_PORT,KEYIN1|KEYIN0);         //clear PT1 interrupt flag
  MCUSTATUSbits._byte = 0;
  PT1INTSTATUSbits._byte = 0;


  InitalI2C();
  SYS_EnableGIE(4,0x3F);                                // Enable GIE(Global Interrupt)
  DisplayInit();
  ClearLCDframe();
  LCD_DATA_DISPLAY(count);

  while(1)
  {
    if(PT1INTSTATUSbits.b_PTINT6Done)                   //if PT1.6 low
    {
      LCD_DATA_DISPLAY(count++);
      PT1INTSTATUSbits.b_PTINT6Done=0;
    }
    if(PT1INTSTATUSbits.b_PTINT7Done)                   //if PT1.7 low
    {
      LCD_DATA_DISPLAY(count--);
      PT1INTSTATUSbits.b_PTINT7Done=0;
    }
  }

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW0_ISR()                                                   */
/* Description  : I2C/UART/SPI interrupt Service Routine (HW0).               */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW0_ISR(void)
{
  unsigned char I2C_Status,I2C_IntFlag;

  I2C_IntFlag=DrvI2C_ReadIntFlag();
  if((I2C_IntFlag == E_DRVI2C_INT)||(I2C_IntFlag == E_DRVI2C_INT_ALL))  //Get I2C Interrupt Flag
  {
    I2C_Status=DrvI2C_GetStatusFlag();     //Get I2C Status Flag
    switch(I2C_Status)
    {
      case 0x90:  //MACTFlag+RWFlag
          {       //START has been transmitted
            DrvI2C_WriteData(I2C_TARGET);  //Send Slave Address & R/W Bit
            DrvI2C_Ctrl(0,0,0,0);          //Clear all I2C flag
            break;
          };
      case 0x84:  //MACTFlag+ACKFlag
          {       //Slave A + W has been transmitted. ACK has been received.
            DrvI2C_WriteData(I2C_Sendbuf[I2C_DataTxIndex++]);  //Send Data to Slave
            DrvI2C_Ctrl(0,0,0,0);      //Clear all I2C flag
            break;
          };
      case 0x80: //MACTFlag
          {      //Slave A + W has been transmitted. ACK has been received.
            DrvI2C_WriteData(I2C_Sendbuf[I2C_DataTxIndex++]);  //Send Data to Slave
            DrvI2C_Ctrl(0,0,0,0);      //Clear all I2C flag
            break;
          };
      case 0x30:
          {
            DrvI2C_Ctrl(0,0,0,0);      //Clear all I2C flag
            I2C_EndFlag=1;
            break;
          };
      case 0x8C:  //MACTFlag+DFFlag+ACKFlag
          {       //DATA has been transmitted and ACK has been received
            if(I2C_DataTxIndex<I2C_DataTxLen)
            {
              DrvI2C_WriteData(I2C_Sendbuf[I2C_DataTxIndex++]);  //Send Data to Slave
              DrvI2C_Ctrl(0,0,0,0);    // Clear all I2C flag
            }
            else
            {
              if(I2C_RW == I2C_WRITE)
              {
                DrvI2C_Ctrl(0,1,0,0);  //I2C as master sends STOP signal
                I2C_EndFlag=1;
              }
              else if(I2C_RW == I2C_READ)
                DrvI2C_Ctrl(1,0,0,0);  //I2C as master sends START signal
              I2C_DataTxIndex=0;
            }
            break;
          };
      case 0x88:  //MACTFlag+DFFlag
          {       //DATA has been transmitted and NACK has been received
            DrvI2C_Ctrl(0,1,0,0);    //I2C as master sends STOP signal
            I2C_DataTxIndex=0;
            I2C_EndFlag=1;
            break;
          };
      case 0xB0:
          {       //A repeated START has been transmitted.
            DrvI2C_WriteData(I2C_TARGET | I2C_READ);  //Send Slave Address & R/W Bit
            DrvI2C_Ctrl(0,0,0,0);    //Clear all I2C flag
            break;
          }
      case 0x94:  //MACTFlag+RWFlag+ACKFlag
          {       //Slave A + R has been transmitted. ACK has been received.
            if(I2C_DataRxLen>1)
            {
              DrvI2C_Ctrl(0,0,0,1);  //Set ACK bit
            }else{
              DrvI2C_Ctrl(0,0,0,0);  //Clear all I2C flag
            }
            break;
          };
      case 0x9C:  //MACTFlag+RWFlag+DFFlag+ACKFlag
          {       //Data byte has been received. ACK has been transmitted.
            I2C_Recbuf[I2C_DataRxIndex++]=DrvI2C_ReadData();
            if((I2C_DataRxLen-1)>I2C_DataRxIndex)
            {
              DrvI2C_Ctrl(0,0,0,1);  //Set ACK bit
            }
            else
            {
              DrvI2C_Ctrl(0,0,0,0);  //Clear all I2C flag
            }
            break;
          };
      case 0x98:  //MACTFlag+RWFlag+DFFlag
          {       //Data byte has been received. NACK has been transmitted.
            I2C_Recbuf[I2C_DataRxIndex++]=DrvI2C_ReadData();
            DrvI2C_Ctrl(0,1,0,0);  //I2C as master sends STOP signal
            I2C_EndFlag=1;
            break;
          };
      default:
          {
            DrvI2C_Ctrl(0,0,0,0);  //Clear all I2C flag
            I2C_EndFlag=1;
            break;
          };
    }
    DrvI2C_ClearIRQ();
    DrvI2C_ClearEIRQ();            //Clear EIRQFlag
    DrvI2C_ClearIntFlag(0);        //Clear I2C Interrupt Flag(I2CIF)
  }
  if((I2C_IntFlag == E_DRVI2C_ERROR_INT)||(I2C_IntFlag == E_DRVI2C_INT_ALL))  //Get I2C Error Interrupt Flag
  {
    I2C_EndFlag=1;
    DrvI2C_ClearIRQ();
    DrvI2C_ClearEIRQ();            //Clear EIRQFlag
    DrvI2C_ClearIntFlag(1);        //Clear I2C Interrupt Flag(I2CEIF)
    DrvI2C_Ctrl(0,0,0,0);          //Clear all I2C flag
  }
  SYS_EnableGIE(4,0x3F);           //Enable GIE(Global Interrupt)

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW1_ISR()                                                   */
/* Description  : WDT & RTC & Timer A/B/C interrupt Service Routine (HW1).    */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW1_ISR(void)
{

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW2_ISR()                                                   */
/* Description  : ADC interrupt Service Routine (HW2).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW2_ISR(void)
{

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW3_ISR()                                                   */
/* Description  : CMP & OPA interrupt Service Routine (HW3).                  */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW3_ISR(void)
{

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW4_ISR()                                                   */
/* Description  : PT1 interrupt Service Routine (HW4).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW4_ISR(void)
{
  uint32_t PORT_IntFlag;

  PORT_IntFlag=DrvGPIO_GetIntFlag(KEY_PORT);
  if((PORT_IntFlag&KEYIN0)==KEYIN0)
  {
    PT1INTSTATUSbits.b_PTINT6Done=1;
  }
  if((PORT_IntFlag&KEYIN1)==KEYIN1)
  {
    PT1INTSTATUSbits.b_PTINT7Done=1;
  }
  DrvGPIO_ClearIntFlag(KEY_PORT,KEYIN1|KEYIN0);         //clear PT1_7/1_6 interrupt flag
}

/*----------------------------------------------------------------------------*/
/* Function Name: HW5_ISR()                                                   */
/* Description  : PT2 interrupt Service Routine (HW5).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW5_ISR(void)
{

}

/*----------------------------------------------------------------------------*/
/* Function Name: tlb_exception_handler()                                     */
/* Description  : Exception Service Routines.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void tlb_exception_handler()
{
  asm("nop"); //procedure define by customer.
  asm("nop");
}

/*----------------------------------------------------------------------------*/
/* Software Delay Subroutines                                                 */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
  for(;num>0;num--)
  asm("NOP");
}

/*----------------------------------------------------------------------------*/
/* Function Name: InitalI2C(void)                                             */
/* Description  : Hardware I2C Initial.                                       */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void InitalI2C(void)
{
  DrvI2C_SetIOPin(4);    //setting io pin, 4 SCL=PT2.0;SDA=PT2.1
  DrvI2C_Open(0x4);      //Enable I2C function and set I2C baud rate=100kHz
  //Default CPU clock is 2MHz, Data Baud Rate : (I2CLK)/[4x(CRG+1)]= 2000000/[4*(4+1)]=100kHz
  DrvI2C_EnableInt(2);   //Enable I2C interrupt and error interrupt
}

/*---------------------------------------------------------------------------*/
/* End Of File                                                               */
/*---------------------------------------------------------------------------*/
