/*******************************************************************************
 * DrvHY2613.c
 * -----------------------------------------------------------------------------
 *
 * Copyright (c) 2016-2026 HYCON Technology, Inc.
 * All rights reserved.
 * HYCON Technology <www.hycontek.com>
 *
 * HYCON reserves the right to amend this code without notice at any time.
 * HYCON assumes no responsibility for any errors appeared in the code,
 * and HYCON disclaims any express or implied warranty, relating to sale
 * and/or use of this code including liability or warranties relating
 * to fitness for a particular purpose, or infringement of any patent,
 * copyright or other intellectual property right.
 *
 * * ---------------------------------------------------------------------------
 * Project Name : 
 * IDE tooling  : 
 * Device Ver.  : 
 * Library Ver. : 
 * MCU Device   : 
 * Description  : 
 * Created Date : 2018/2/18
 * Created by   :
 *
 * -----------------------------------------------------------------------------
 * Program Description:
 * 
 ******************************************************************************/

/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include "DrvI2C.h"
#include "HY2613.h"

/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#define I2CBufferSize   64
#define I2C_WRITE       0x00           // I2C WRITE command
#define I2C_READ        0x01           // I2C READ command
#define I2C_Delay       1000000        //HAO=2MHz, wait 5s
/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
extern unsigned char I2C_RW;
extern unsigned char I2C_TARGET;       // Target I2C slave address
extern unsigned char I2C_EndFlag;
extern unsigned int  I2C_Sendbuf[I2CBufferSize];
extern unsigned int  I2C_Recbuf[I2CBufferSize];
extern unsigned int  I2C_DataTxLen,I2C_DataTxIndex,I2C_DataRxLen,I2C_DataRxIndex;
unsigned int I2C_Timeout;

/*----------------------------------------------------------------------------*/
/* Function Name: DisplayInit(void)                                           */
/* Description  : Inital the LCD Drive.                                       */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void DisplayInit(void)
{

  I2C_TARGET  = HY2613_Slave_addr;                 //Slave_addr equ  07Ch
  I2C_Sendbuf[0]  = ICSET|SWRst|OscModeInt;        //ICSET equ  0EAh
  I2C_Sendbuf[1]  = DISCTL|PoMode3|FrInv|PoHigh;   //DISCTL equ  0BFh
  I2C_Sendbuf[2]  = ADSET;                         //ADSET equ  000h
  I2C_Sendbuf[3]  = ADSET;                         //ADSET equ  000h
  I2C_RW = I2C_WRITE;
  I2C_Timeout=I2C_Delay;  //HAO=2MHz, wait 5s
  I2C_DataTxLen=4;
  I2C_DataTxIndex=0;
  I2C_EndFlag=0;
  DrvI2C_Ctrl(1,0,0,0);                            // I2C as master sends START signal
  asm("NOP");
  if(I2C_EndFlag == 0)                             // Wait I2C Tx Finish
  {
    while((I2C_Timeout != 0) && (I2C_EndFlag==0) )
    {
	  I2C_Timeout--;
	  asm("nop");
    }
  }

/*----------------------------------------------------------------------------*/
/* Extend Control                                                             */
/*----------------------------------------------------------------------------*/
#if defined(HY2613C)
  I2C_TARGET  = HY2613_Slave_addr;                 //Slave_addr equ  07Ch
  I2C_Sendbuf[0]  = EXCTL|SROn;
  I2C_RW = I2C_WRITE;
  I2C_Timeout=I2C_Delay;  //HAO=2MHz, wait 5s
  I2C_DataTxLen=1;
  I2C_DataTxIndex=0;
  I2C_EndFlag=0;
  DrvI2C_Ctrl(1,0,0,0);                            // I2C as master sends START signal
  asm("NOP");
  if(I2C_EndFlag == 0)                             // Wait I2C Tx Finish
  {
    while((I2C_Timeout != 0) && (I2C_EndFlag==0) )
    {
	  I2C_Timeout--;
	  asm("nop");
    }
  }

  I2C_TARGET  = HY2613_Slave_addr;                 //Slave_addr equ  07Ch
  I2C_Sendbuf[0]  = VLCD_CTVR;
  I2C_Sendbuf[1]  = VLCD3p0|CTVR00;
  I2C_Sendbuf[2]  = ENBUF|VLCDEN01;
  I2C_RW = I2C_WRITE;
  I2C_Timeout=I2C_Delay;  //HAO=2MHz, wait 5s
  I2C_DataTxLen=3;
  I2C_DataTxIndex=0;
  I2C_EndFlag=0;
  DrvI2C_Ctrl(1,0,0,0);                            // I2C as master sends START signal
  asm("NOP");
  if(I2C_EndFlag == 0)                             // Wait I2C Tx Finish
  {
    while((I2C_Timeout != 0) && (I2C_EndFlag==0) )
    {
	  I2C_Timeout--;
	  asm("nop");
    }
  }

  I2C_TARGET  = HY2613_Slave_addr;                 //Slave_addr equ  07Ch
  I2C_Sendbuf[0]  = EXCTL;
  I2C_RW = I2C_WRITE;
  I2C_Timeout=I2C_Delay;  //HAO=2MHz, wait 5s
  I2C_DataTxLen=1;
  I2C_DataTxIndex=0;
  I2C_EndFlag=0;
  DrvI2C_Ctrl(1,0,0,0);                            // I2C as master sends START signal
  asm("NOP");
  if(I2C_EndFlag == 0)                             // Wait I2C Tx Finish
  {
    while((I2C_Timeout != 0) && (I2C_EndFlag==0) )
    {
	  I2C_Timeout--;
	  asm("nop");
    }
  }
#endif
/*----------------------------------------------------------------------------*/
/* Open the LCD Drive                                                         */
/*----------------------------------------------------------------------------*/
  I2C_TARGET  = HY2613_Slave_addr;                 //Slave_addr 0x07C
  I2C_Sendbuf[0]  = DISCTL|PoMode3|FrInv|PoHigh;
  I2C_Sendbuf[1]  = BLKCTL;                        //0xF0
  I2C_Sendbuf[2]  = PIXCTL;                        //0xFC
  I2C_Sendbuf[3]  = MODE_SET|Dis_ON;               //0xC8
  I2C_RW = I2C_WRITE;
  I2C_Timeout=I2C_Delay;  //HAO=2MHz, wait 5s
  I2C_DataTxLen=4;
  I2C_DataTxIndex=0;
  I2C_EndFlag=0;
  DrvI2C_Ctrl(1,0,0,0);                            // I2C as master sends START signal
  asm("NOP");
  if(I2C_EndFlag == 0)                             // Wait I2C Tx Finish
  {
    while((I2C_Timeout != 0) && (I2C_EndFlag==0) )
    {
	  I2C_Timeout--;
	  asm("nop");
    }
  }
}

/*----------------------------------------------------------------------------*/
/* Function Name: RAM2LCD(unsigned char *Buffer_Adr, unsigned int length)     */
/* Description  : RAM Data Send to LCD .                                      */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void RAM2LCD(unsigned char *Buffer_Adr, unsigned int length)
{
  I2C_TARGET  = HY2613_Slave_addr;                 //Slave_addr equ  07Ch
  I2C_Sendbuf[0]  = DISCTL|PoMode3|FrInv|PoHigh;
  I2C_Sendbuf[1]  = BLKCTL;                        //0xf0
  I2C_Sendbuf[2]  = PIXCTL;                        //0xfc
  I2C_Sendbuf[3]  = MODE_SET|Dis_ON;               //0xc8
  I2C_Sendbuf[4]  = ADSET;                         //0x00 LCD RAM
  I2C_RW = I2C_WRITE;
  I2C_Timeout=I2C_Delay;  //HAO=2MHz, wait 5s
  I2C_DataTxLen=5;
  for(;length>0;length--)
  {
    I2C_Sendbuf[I2C_DataTxLen++]  = (*Buffer_Adr++);
  }
  I2C_DataTxIndex=0;
  I2C_EndFlag=0;
  DrvI2C_Ctrl(1,0,0,0);                            // I2C as master sends START signal
  asm("NOP");
  if(I2C_EndFlag == 0)                             // Wait I2C Tx Finish
  {
    while((I2C_Timeout != 0) && (I2C_EndFlag==0) )
    {
	  I2C_Timeout--;
	  asm("nop");
    }
  }
}
