/*******************************************************************************
 *
 * Copyright (c) 2016-2026 HYCON Technology, Inc.
 * All rights reserved.
 * HYCON Technology <www.hycontek.com>
 *
 * HYCON reserves the right to amend this code without notice at any time.
 * HYCON assumes no responsibility for any errors appeared in the code,
 * and HYCON disclaims any express or implied warranty, relating to sale
 * and/or use of this code including liability or warranties relating
 * to fitness for a particular purpose, or infringement of any patent,
 * copyright or other intellectual property right.
 *
 * -----------------------------------------------------------------------------
 * Project Name : HY16F188_RTC
 * IDE tooling  : AndeSight C/C++ IDE, version: 2.1.1 Build ID : 201608241332
 * Device Ver.  : HY16F_RDSp3_DeviceV0.2 crt0.o for HY16F18x MCU.
 * Library Ver. : 1.5
 * MCU Device   : 
 * Description  : 
 * Created Date : 2018/2/18
 * Created by   :
 *
 * Program Description:
 * --------------------
 *
 * ---------
 * HY16F188 |              --------------------
 *    PT2.0 | SCL ---> SCL | LCD Drive HY2613 |
 *    PT2.1 | SDA ---> SDA --------------------
 *          |
 *    PT2.4 | LXIN
 *    PT2.5 | LXOUT
 *     GND  |
 *          |
 * ---------
 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include "HY16F188.h"
#include "System.h"
#include "DrvGPIO.h"
#include "DrvI2C.h"
#include "DrvCLOCK.h"
#include "DrvRTC.h"
#include "HY2613.h"
#include "my define.h"

/*----------------------------------------------------------------------------*/
/* STRUCTURES                                                                 */
/*----------------------------------------------------------------------------*/
volatile typedef union _MCUSTATUS
{
  char  _byte;
  struct
  {
    unsigned b_ADCdone:1;
    unsigned b_TMAdone:1;
    unsigned b_TMBdone:1;
    unsigned b_TMC0done:1;
    unsigned b_TMC1done:1;
    unsigned b_RTCdone:1;
    unsigned b_UART_TxDone:1;
    unsigned b_UART_RxDone:1;
  };
} MCUSTATUS;


/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#define  I2CBufferSize  64
#define I2C_WRITE       0x00           // I2C WRITE command
#define I2C_READ        0x01           // I2C READ command
//#define  RTC_counter_show
#define  TimerDtata_show

/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
unsigned char I2C_RW;
unsigned char I2C_TARGET;
unsigned char I2C_EndFlag;
unsigned int I2C_Sendbuf[I2CBufferSize];
unsigned int I2C_Recbuf[I2CBufferSize];
unsigned int I2C_DataTxLen,I2C_DataTxIndex,I2C_DataRxLen,I2C_DataRxIndex;

MCUSTATUS  MCUSTATUSbits;
unsigned int sec,min,hour,week,day,month,year ;
unsigned int RTC_counter;
unsigned int TimerDtata;
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);
void InitalI2C(void);
void InitalRTC(void);
int ComputeWeek(int TempYear, int TempMonth, int TempDay);
/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
int main(void)
{

  S_DRVRTC_TIME_DATA_T sCurTime;        //Setting Start
  RTC_counter=0;
  TimerDtata=0;

  InitalI2C();

  SYS_EnableGIE(4,0x3F);                // Enable GIE(Global Interrupt)

  DisplayInit();
  ClearLCDframe();
  Delay(10000);
  DisplayHYcon();
  Delay(10000);
  MCUSTATUSbits._byte = 0;

  InitalRTC();


  while(1)
  {
    if(MCUSTATUSbits.b_RTCdone==1)
    {
      DrvRTC_Read(DRVRTC_CURRENT_TIME,&sCurTime);
      TimerDtata=sCurTime.u32cSecond+sCurTime.u32cMinute*100+sCurTime.u32cHour*10000;
#if defined(TimerDtata_show)
      LCD_DATA_DISPLAY(TimerDtata);
#endif
#if defined(RTC_counter_show)
      LCD_DATA_DISPLAY(RTC_counter);
#endif

      sec=sCurTime.u32cSecond;
      min=sCurTime.u32cMinute;
      hour=sCurTime.u32cHour;
      week=sCurTime.u32cDayOfWeek;
      day=sCurTime.u32cDay;
      month=sCurTime.u32cMonth;
      year=sCurTime.u32Year;
      MCUSTATUSbits.b_RTCdone=0;
    }
  }

  return 0;

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW0_ISR()                                                   */
/* Description  : I2C/UART/SPI interrupt Service Routine (HW0).               */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW0_ISR(void)
{
  unsigned char I2C_Status,I2C_IntFlag;

  I2C_IntFlag=DrvI2C_ReadIntFlag();
  if((I2C_IntFlag == E_DRVI2C_INT)||(I2C_IntFlag == E_DRVI2C_INT_ALL))  //Get I2C Interrupt Flag
  {
    I2C_Status=DrvI2C_GetStatusFlag();     //Get I2C Status Flag
    switch(I2C_Status)
    {
      case 0x90:  //MACTFlag+RWFlag
          {       //START has been transmitted
            DrvI2C_WriteData(I2C_TARGET);  //Send Slave Address & R/W Bit
            DrvI2C_Ctrl(0,0,0,0);          //Clear all I2C flag
            break;
          };
      case 0x84:  //MACTFlag+ACKFlag
          {       //Slave A + W has been transmitted. ACK has been received.
            DrvI2C_WriteData(I2C_Sendbuf[I2C_DataTxIndex++]);  //Send Data to Slave
            DrvI2C_Ctrl(0,0,0,0);      //Clear all I2C flag
            break;
          };
      case 0x80: //MACTFlag
          {      //Slave A + W has been transmitted. ACK has been received.
            DrvI2C_WriteData(I2C_Sendbuf[I2C_DataTxIndex++]);  //Send Data to Slave
            DrvI2C_Ctrl(0,0,0,0);      //Clear all I2C flag
            break;
          };
      case 0x30:
          {
            DrvI2C_Ctrl(0,0,0,0);      //Clear all I2C flag
            I2C_EndFlag=1;
            break;
          };
      case 0x8C:  //MACTFlag+DFFlag+ACKFlag
          {       //DATA has been transmitted and ACK has been received
            if(I2C_DataTxIndex<I2C_DataTxLen)
            {
              DrvI2C_WriteData(I2C_Sendbuf[I2C_DataTxIndex++]);  //Send Data to Slave
              DrvI2C_Ctrl(0,0,0,0);    // Clear all I2C flag
            }
            else
            {
              if(I2C_RW == I2C_WRITE)
              {
                DrvI2C_Ctrl(0,1,0,0);  //I2C as master sends STOP signal
                I2C_EndFlag=1;
              }
              else if(I2C_RW == I2C_READ)
                DrvI2C_Ctrl(1,0,0,0);  //I2C as master sends START signal
              I2C_DataTxIndex=0;
            }
            break;
          };
      case 0x88:  //MACTFlag+DFFlag
          {       //DATA has been transmitted and NACK has been received
            DrvI2C_Ctrl(0,1,0,0);    //I2C as master sends STOP signal
            I2C_DataTxIndex=0;
            I2C_EndFlag=1;
            break;
          };
      case 0xB0:
          {       //A repeated START has been transmitted.
            DrvI2C_WriteData(I2C_TARGET | I2C_READ);  //Send Slave Address & R/W Bit
            DrvI2C_Ctrl(0,0,0,0);    //Clear all I2C flag
            break;
          }
      case 0x94:  //MACTFlag+RWFlag+ACKFlag
          {       //Slave A + R has been transmitted. ACK has been received.
            if(I2C_DataRxLen>1)
            {
              DrvI2C_Ctrl(0,0,0,1);  //Set ACK bit
            }else{
              DrvI2C_Ctrl(0,0,0,0);  //Clear all I2C flag
            }
            break;
          };
      case 0x9C:  //MACTFlag+RWFlag+DFFlag+ACKFlag
          {       //Data byte has been received. ACK has been transmitted.
            I2C_Recbuf[I2C_DataRxIndex++]=DrvI2C_ReadData();
            if((I2C_DataRxLen-1)>I2C_DataRxIndex)
            {
              DrvI2C_Ctrl(0,0,0,1);  //Set ACK bit
            }
            else
            {
              DrvI2C_Ctrl(0,0,0,0);  //Clear all I2C flag
            }
            break;
          };
      case 0x98:  //MACTFlag+RWFlag+DFFlag
          {       //Data byte has been received. NACK has been transmitted.
            I2C_Recbuf[I2C_DataRxIndex++]=DrvI2C_ReadData();
            DrvI2C_Ctrl(0,1,0,0);  //I2C as master sends STOP signal
            I2C_EndFlag=1;
            break;
          };
      default:
          {
            DrvI2C_Ctrl(0,0,0,0);  //Clear all I2C flag
            I2C_EndFlag=1;
            break;
          };
    }
    DrvI2C_ClearIRQ();
    DrvI2C_ClearEIRQ();            //Clear EIRQFlag
    DrvI2C_ClearIntFlag(0);        //Clear I2C Interrupt Flag(I2CIF)
  }
  if((I2C_IntFlag == E_DRVI2C_ERROR_INT)||(I2C_IntFlag == E_DRVI2C_INT_ALL))  //Get I2C Error Interrupt Flag
  {
    I2C_EndFlag=1;
    DrvI2C_ClearIRQ();
    DrvI2C_ClearEIRQ();            //Clear EIRQFlag
    DrvI2C_ClearIntFlag(1);        //Clear I2C Interrupt Flag(I2CEIF)
    DrvI2C_Ctrl(0,0,0,0);          //Clear all I2C flag
  }
  SYS_EnableGIE(4,0x3F);           //Enable GIE(Global Interrupt)

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW1_ISR()                                                   */
/* Description  : WDT & RTC & Timer A/B/C interrupt Service Routine (HW1).    */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW1_ISR(void)
{
  if(DrvRTC_ReadState()&&0x2)  //check PTF flag
  {
    DrvRTC_ClearState(E_DRVRTC_CLEAR_ALL);
    DrvRTC_ClearIntFlag();
    RTC_counter++;
    MCUSTATUSbits.b_RTCdone=1;
  }
}

/*----------------------------------------------------------------------------*/
/* Function Name: HW2_ISR()                                                   */
/* Description  : ADC interrupt Service Routine (HW2).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW2_ISR(void)
{

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW3_ISR()                                                   */
/* Description  : CMP & OPA interrupt Service Routine (HW3).                  */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW3_ISR(void)
{

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW4_ISR()                                                   */
/* Description  : PT1 interrupt Service Routine (HW4).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW4_ISR(void)
{

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW5_ISR()                                                   */
/* Description  : PT2 interrupt Service Routine (HW5).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW5_ISR(void)
{

}

/*----------------------------------------------------------------------------*/
/* Function Name: tlb_exception_handler()                                     */
/* Description  : Exception Service Routines.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void tlb_exception_handler()
{
  asm("nop"); //procedure define by customer.
  asm("nop");
}

/*----------------------------------------------------------------------------*/
/* Software Delay Subroutines                                                 */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
  for(;num>0;num--)
  asm("NOP");
}

/*----------------------------------------------------------------------------*/
/* Function Name: InitalI2C(void)                                             */
/* Description  : Hardware I2C Initial.                                       */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void InitalI2C(void)
{
  DrvI2C_SetIOPin(4);    //setting io pin, 4 SCL=PT2.0;SDA=PT2.1
  DrvI2C_Open(0x4);      //Enable I2C function and set I2C baud rate=100kHz
  //Default CPU clock is 2MHz, Data Baud Rate : (I2CLK)/[4x(CRG+1)]= 2000000/[4*(4+1)]=100kHz
  DrvI2C_EnableInt(2);   //Enable I2C interrupt and error interrupt
}

/*----------------------------------------------------------------------------*/
/* Function Name: InitalRTC()                                                 */
/* Description  : RTC Initialization Subroutines .                            */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void InitalRTC()
{
  S_DRVRTC_TIME_DATA_T sCurTime;        //Setting Start
  DrvRTC_ClkConfig(1);
  DrvCLOCK_EnableLowOSC(E_EXTERNAL,130000);
  DrvRTC_ClockSource(0);                //RTC clock source =LSXT

  //DRVRTC_CURRENT_TIME or Alarm Time
  DrvRTC_WriteEnable();
  DrvRTC_Read(DRVRTC_CURRENT_TIME,&sCurTime);
  sCurTime.u8cClockDisplay=0;           //DRVRTC_CLOCK_12//DRVRTC_CLOCK_24
  sCurTime.u8cAmPm=0;                   //DRVRTC_AM//DRVRTC_PM
  sCurTime.u32cSecond=40;
  sCurTime.u32cMinute=59;
  sCurTime.u32cHour=23;
  sCurTime.u32cDay=18;
  sCurTime.u32cMonth=2;
  sCurTime.u32Year=2018;
  sCurTime.u32cDayOfWeek=ComputeWeek(sCurTime.u32Year,sCurTime.u32cMonth,sCurTime.u32cDay);
  sCurTime.u8IsEnableWakeUp=0;          //WK
  DrvRTC_Write(DRVRTC_CURRENT_TIME,&sCurTime);
  DrvRTC_HourFormat(E_DRVRTC_HOUR_24);  //1:12hour 0:24hour
  DrvRTC_Enable();
  DrvRTC_PeriodicTimeEnable(E_DRVRTC_1_8_SEC);
  DrvRTC_ClearIntFlag();
  DrvRTC_EnableInt();                   //Enable RTC interrupt
}

/*----------------------------------------------------------------------------*/
/* Function Name: ComputeWeek(int TempYear, int TempMonth, int TempDay)       */
/* Description  : Compute Week Subroutines.                                   */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
int ComputeWeek(int TempYear, int TempMonth, int TempDay)
{
  int TempWeek;
  if (TempMonth >= 3)
  {
    TempMonth = TempMonth - 2;
  }
  else
  {
    TempMonth = TempMonth + 10;
    TempYear--;
  }

  TempWeek = TempYear + (int)(TempYear / 4) - (int)(TempYear / 100) +(int)(TempYear / 400) + (int)(2.6 * TempMonth - 0.2) + TempDay;
  TempWeek = TempWeek - 7*(int)(TempWeek / 7);
  return (TempWeek);
}

/*---------------------------------------------------------------------------*/
/* End Of File                                                               */
/*---------------------------------------------------------------------------*/
