/*******************************************************************************
 *
 * Copyright (c) 2016-2026 HYCON Technology, Inc.
 * All rights reserved.
 * HYCON Technology <www.hycontek.com>
 *
 * HYCON reserves the right to amend this code without notice at any time.
 * HYCON assumes no responsibility for any errors appeared in the code,
 * and HYCON disclaims any express or implied warranty, relating to sale
 * and/or use of this code including liability or warranties relating
 * to fitness for a particular purpose, or infringement of any patent,
 * copyright or other intellectual property right.
 *
 * -----------------------------------------------------------------------------
 * Project Name : HY16F188_UART
 * IDE tooling  : AndeSight C/C++ IDE, version: 2.1.1 Build ID : 201608241332
 * Device Ver.  : HY16F_RDSp3_DeviceV0.2 crt0.o for HY16F18x MCU.
 * Library Ver. : 1.5
 * MCU Device   : 
 * Description  : 
 * Created Date : 2018/2/18
 * Created by   :
 *
 * Program Description:
 * --------------------
 *
 * ---------
 * HY16F188 |              --------------------
 *    PT2.0 | SCL ---> SCL | LCD Drive HY2613 |
 *    PT2.1 | SDA ---> SDA --------------------
 *    PT2.2 | TX  --->
 *    PT2.3 | RX  <---
 *     GND  |
 *          |
 * ---------
 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include "HY16F188.h"
#include "System.h"
#include "DrvGPIO.h"
#include "DrvUART.h"
#include "DrvI2C.h"
#include "DrvCLOCK.h"
#include "HY2613.h"
#include "my define.h"

/*----------------------------------------------------------------------------*/
/* STRUCTURES                                                                 */
/*----------------------------------------------------------------------------*/
volatile typedef union _MCUSTATUS
{
  char  _byte;
  struct
  {
    unsigned b_ADCdone:1;
    unsigned b_TMAdone:1;
    unsigned b_TMBdone:1;
    unsigned b_TMC0done:1;
    unsigned b_TMC1done:1;
    unsigned b_RTCdone:1;
    unsigned b_UART_TxDone:1;
    unsigned b_UART_RxDone:1;
  };
} MCUSTATUS;


/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#define  I2CBufferSize  64
#define I2C_WRITE       0x00           // I2C WRITE command
#define I2C_READ        0x01           // I2C READ command

#define HSRC  //Internal HAO
//#define HSXT  //External 4MHz

#define HAO_2MHZ
//#define HAO_4MHZ
//#define HAO_10MHZ

#define  UART_PORT E_PT2
#define  UART_TXD  BIT2
#define  UART_RXD  BIT3
#define  UartBufferSize 128
#define  Uart_RX_BufferSize 6
#define  Uart_TX_BufferSize 8
/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
unsigned char I2C_RW;
unsigned char I2C_TARGET;
unsigned char I2C_EndFlag;
unsigned int I2C_Sendbuf[I2CBufferSize];
unsigned int I2C_Recbuf[I2CBufferSize];
unsigned int I2C_DataTxLen,I2C_DataTxIndex,I2C_DataRxLen,I2C_DataRxIndex;

MCUSTATUS  MCUSTATUSbits;

unsigned char UartRxBuffer[Uart_RX_BufferSize]={0},UartTxBuffer[Uart_TX_BufferSize]={0};
unsigned char UartTxIndex,UartTxLength,UartRxIndex,UartRxLength;
unsigned char UartRxBuffer_Command[Uart_RX_BufferSize]=
{
0x61,0x62,0x63,0x64,0x65,0x66  //ASCII  KEYWORD = abcdef
};
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);
void InitalI2C(void);
void InitalUart(void);
void InitalClock(void);
/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
int main(void)
{

  unsigned char Stop_To_Send_UART=DISABLE;
  InitalClock();
  InitalI2C();
  InitalUart();

  MCUSTATUSbits._byte = 0;
  MCUSTATUSbits.b_UART_TxDone=ENABLE;
  SYS_EnableGIE(4,0x3F);  //Enable GIE(Global Interrupt)

  DisplayInit();
  ClearLCDframe();
  Delay(10000);
  DisplayHYcon();
  Delay(50000);

  UartTxIndex=0;
  UartRxIndex=0;

  while(1)
  {

    //UART RX
    if(MCUSTATUSbits.b_UART_RxDone==ENABLE)
    {

      if(
        (UartRxBuffer[5]==UartRxBuffer_Command[5] && UartRxBuffer[4]==UartRxBuffer_Command[4]) &&
        (UartRxBuffer[3]==UartRxBuffer_Command[3] && UartRxBuffer[2]==UartRxBuffer_Command[2]) &&
        (UartRxBuffer[1]==UartRxBuffer_Command[1] && UartRxBuffer[0]==UartRxBuffer_Command[0])
        )
    {
        //if UART receive == 0x61(a)0x62(b)0x63(c)0x64(d)0x65(e)0x66(f)
        //send out 0x61(a)0x62(b)0x63(c)0x64(d)0x65(e)0x66(f) and stop to send out
        Stop_To_Send_UART=ENABLE;  //UART stop to send out ABCDEF
        for(UartTxLength=0;UartTxLength<=Uart_TX_BufferSize;UartTxLength++)
        {
          UartTxBuffer[UartTxLength]=UartRxBuffer[UartTxLength];
          if(UartTxLength==Uart_RX_BufferSize)
          {
            UartRxIndex=0;
            MCUSTATUSbits.b_UART_TxDone=DISABLE;
            DrvUART_EnableInt(ENABLE,DISABLE);  //Enable UART Tx Interrupt, Disable UART Rx Interrup
            while(!MCUSTATUSbits.b_UART_TxDone);  //If  MCUSTATUSbits.b_UART_TxDone=DISABLE, stop at here
          }
        }
    }
    else
    {
      //if UART receive != 0x61(a)0x62(b)0x63(c)0x64(d)0x65(e)0x66(f)
      //User defined at here
      Stop_To_Send_UART=DISABLE;  //UART start to send out ABCDEF
    }
      UartRxIndex=0;  //When finished the data reception. Set UartRxIndex=0
      MCUSTATUSbits.b_UART_RxDone=DISABLE;
    }

    //UART TX
    if(MCUSTATUSbits.b_UART_TxDone==ENABLE && Stop_To_Send_UART==DISABLE )
    {
      UartTxBuffer[7]='\r';
      UartTxBuffer[6]='\n';
      UartTxBuffer[5]=0x46;  //F
      UartTxBuffer[4]=0x45;  //E
      UartTxBuffer[3]=0x44;  //D
      UartTxBuffer[2]=0x43;  //C
      UartTxBuffer[1]=0x42;  //B
      UartTxBuffer[0]=0x41;  //A
      MCUSTATUSbits.b_UART_TxDone=DISABLE;
      UartTxLength=8;
      UartTxIndex=0;
      DrvUART_EnableInt(ENABLE,ENABLE);  //Enable UART Tx Interrupt;Enable UART Rx Interrupt
      while(!MCUSTATUSbits.b_UART_TxDone);  //If  MCUSTATUSbits.b_UART_TxDone=DISABLE, stop at here
    }

  }


}
/*----------------------------------------------------------------------------*/
/* Function Name: HW0_ISR()                                                   */
/* Description  : I2C/UART/SPI interrupt Service Routine (HW0).               */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW0_ISR(void)
{
  unsigned char I2C_Status,I2C_IntFlag;

  if(DrvUART_GetRxFlag())
  {
    UartRxBuffer[UartRxIndex]=DrvUART_Read();
    UartRxIndex++;
    if(UartRxIndex>=Uart_RX_BufferSize)
    {
      UartRxIndex=0;
      MCUSTATUSbits.b_UART_RxDone=ENABLE;
    }
      DrvUART_ClrRxFlag();
  }

  if(DrvUART_GetTxFlag())
  {
    if(MCUSTATUSbits.b_UART_TxDone==DISABLE)
    {
      DrvUART_Write(UartTxBuffer[UartTxIndex++]);
      DrvUART_ClrTxFlag();
      if(UartTxIndex>=UartTxLength)
      {
        DrvUART_EnableInt(ENABLE,ENABLE);  //ENABLE UART Tx Interrupt, ENABLE UART Rx Interrupt
        MCUSTATUSbits.b_UART_TxDone=ENABLE;
        UartTxIndex=0;
      }
    }
    if(MCUSTATUSbits.b_UART_TxDone==ENABLE)
    {
      DrvUART_EnableInt(DISABLE,ENABLE);  //DISABLE UART Tx Interrupt, ENABLE UART Rx Interrupt
      DrvUART_ClrTxFlag();
    }
  }

  I2C_IntFlag=DrvI2C_ReadIntFlag();
  if((I2C_IntFlag == E_DRVI2C_INT)||(I2C_IntFlag == E_DRVI2C_INT_ALL))  //Get I2C Interrupt Flag
  {
    I2C_Status=DrvI2C_GetStatusFlag();     //Get I2C Status Flag
    switch(I2C_Status)
    {
      case 0x90:  //MACTFlag+RWFlag
          {       //START has been transmitted
            DrvI2C_WriteData(I2C_TARGET);  //Send Slave Address & R/W Bit
            DrvI2C_Ctrl(0,0,0,0);          //Clear all I2C flag
            break;
          };
      case 0x84:  //MACTFlag+ACKFlag
          {       //Slave A + W has been transmitted. ACK has been received.
            DrvI2C_WriteData(I2C_Sendbuf[I2C_DataTxIndex++]);  //Send Data to Slave
            DrvI2C_Ctrl(0,0,0,0);      //Clear all I2C flag
            break;
          };
      case 0x80: //MACTFlag
          {      //Slave A + W has been transmitted. ACK has been received.
            DrvI2C_WriteData(I2C_Sendbuf[I2C_DataTxIndex++]);  //Send Data to Slave
            DrvI2C_Ctrl(0,0,0,0);      //Clear all I2C flag
            break;
          };
      case 0x30:
          {
            DrvI2C_Ctrl(0,0,0,0);      //Clear all I2C flag
            I2C_EndFlag=1;
            break;
          };
      case 0x8C:  //MACTFlag+DFFlag+ACKFlag
          {       //DATA has been transmitted and ACK has been received
            if(I2C_DataTxIndex<I2C_DataTxLen)
            {
              DrvI2C_WriteData(I2C_Sendbuf[I2C_DataTxIndex++]);  //Send Data to Slave
              DrvI2C_Ctrl(0,0,0,0);    // Clear all I2C flag
            }
            else
            {
              if(I2C_RW == I2C_WRITE)
              {
                DrvI2C_Ctrl(0,1,0,0);  //I2C as master sends STOP signal
                I2C_EndFlag=1;
              }
              else if(I2C_RW == I2C_READ)
                DrvI2C_Ctrl(1,0,0,0);  //I2C as master sends START signal
              I2C_DataTxIndex=0;
            }
            break;
          };
      case 0x88:  //MACTFlag+DFFlag
          {       //DATA has been transmitted and NACK has been received
            DrvI2C_Ctrl(0,1,0,0);    //I2C as master sends STOP signal
            I2C_DataTxIndex=0;
            I2C_EndFlag=1;
            break;
          };
      case 0xB0:
          {       //A repeated START has been transmitted.
            DrvI2C_WriteData(I2C_TARGET | I2C_READ);  //Send Slave Address & R/W Bit
            DrvI2C_Ctrl(0,0,0,0);    //Clear all I2C flag
            break;
          }
      case 0x94:  //MACTFlag+RWFlag+ACKFlag
          {       //Slave A + R has been transmitted. ACK has been received.
            if(I2C_DataRxLen>1)
            {
              DrvI2C_Ctrl(0,0,0,1);  //Set ACK bit
            }else{
              DrvI2C_Ctrl(0,0,0,0);  //Clear all I2C flag
            }
            break;
          };
      case 0x9C:  //MACTFlag+RWFlag+DFFlag+ACKFlag
          {       //Data byte has been received. ACK has been transmitted.
            I2C_Recbuf[I2C_DataRxIndex++]=DrvI2C_ReadData();
            if((I2C_DataRxLen-1)>I2C_DataRxIndex)
            {
              DrvI2C_Ctrl(0,0,0,1);  //Set ACK bit
            }
            else
            {
              DrvI2C_Ctrl(0,0,0,0);  //Clear all I2C flag
            }
            break;
          };
      case 0x98:  //MACTFlag+RWFlag+DFFlag
          {       //Data byte has been received. NACK has been transmitted.
            I2C_Recbuf[I2C_DataRxIndex++]=DrvI2C_ReadData();
            DrvI2C_Ctrl(0,1,0,0);  //I2C as master sends STOP signal
            I2C_EndFlag=1;
            break;
          };
      default:
          {
            DrvI2C_Ctrl(0,0,0,0);  //Clear all I2C flag
            I2C_EndFlag=1;
            break;
          };
    }
    DrvI2C_ClearIRQ();
    DrvI2C_ClearEIRQ();            //Clear EIRQFlag
    DrvI2C_ClearIntFlag(0);        //Clear I2C Interrupt Flag(I2CIF)
  }
  if((I2C_IntFlag == E_DRVI2C_ERROR_INT)||(I2C_IntFlag == E_DRVI2C_INT_ALL))  //Get I2C Error Interrupt Flag
  {
    I2C_EndFlag=1;
    DrvI2C_ClearIRQ();
    DrvI2C_ClearEIRQ();            //Clear EIRQFlag
    DrvI2C_ClearIntFlag(1);        //Clear I2C Interrupt Flag(I2CEIF)
    DrvI2C_Ctrl(0,0,0,0);          //Clear all I2C flag
  }
  SYS_EnableGIE(4,0x3F);           //Enable GIE(Global Interrupt)
}

/*----------------------------------------------------------------------------*/
/* Function Name: HW1_ISR()                                                   */
/* Description  : WDT & RTC & Timer A/B/C interrupt Service Routine (HW1).    */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW1_ISR(void)
{

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW2_ISR()                                                   */
/* Description  : ADC interrupt Service Routine (HW2).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW2_ISR(void)
{

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW3_ISR()                                                   */
/* Description  : CMP & OPA interrupt Service Routine (HW3).                  */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW3_ISR(void)
{

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW4_ISR()                                                   */
/* Description  : PT1 interrupt Service Routine (HW4).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW4_ISR(void)
{

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW5_ISR()                                                   */
/* Description  : PT2 interrupt Service Routine (HW5).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW5_ISR(void)
{

}

/*----------------------------------------------------------------------------*/
/* Function Name: tlb_exception_handler()                                     */
/* Description  : Exception Service Routines.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void tlb_exception_handler()
{
  asm("nop");  //procedure define by customer.
  asm("nop");
}

/*----------------------------------------------------------------------------*/
/* Software Delay Subroutines                                                 */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
  for(;num>0;num--)
  asm("NOP");
}

/*----------------------------------------------------------------------------*/
/* Function Name: InitalI2C(void)                                             */
/* Description  : Hardware I2C Initial.                                       */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void InitalI2C(void)
{

  DrvI2C_SetIOPin(4);    //setting io pin, 4 SCL=PT2.0;SDA=PT2.1
#if defined(HSRC)
  #if defined(HAO_2MHZ)
  DrvI2C_Open(0x4);      //Enable I2C function and set I2C baud rate=100kHz
  //Default CPU clock is 2MHz, Data Baud Rate : (I2CLK)/[4x(CRG+1)]= 2000000/[4*(4+1)]=100kHz
  #endif
  #if defined(HAO_4MHZ)
  DrvI2C_Open(0x9);      //Enable I2C function and set I2C baud rate=100kHz
  //Default CPU clock is 4MHz, Data Baud Rate : (I2CLK)/[4x(CRG+1)]= 4000000/[4*(9+1)]=100kHz
  #endif
  #if defined(HAO_10MHZ)
  DrvI2C_Open(0x18);      //Enable I2C function and set I2C baud rate=100kHz
  //Default CPU clock is 10MHz, Data Baud Rate : (I2CLK)/[4x(CRG+1)]= 10000000/[4*(24+1)]=100kHz
  #endif
#endif
#if defined(HSXT)
  DrvI2C_Open(0x9);      //Enable I2C function and set I2C baud rate=100kHz
  //Default CPU clock is 4MHz, Data Baud Rate : (I2CLK)/[4x(CRG+1)]= 4000000/[4*(9+1)]=100kHz
#endif

  DrvI2C_EnableInt(2);   //Enable I2C interrupt and error interrupt
}

/*----------------------------------------------------------------------------*/
/* Function Name: InitalClock()                                               */
/* Description  : CLOCK Initial Subroutines.                                  */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void InitalClock(void)
{

#if defined(HSRC)
  #if defined(HAO_2MHZ)
  //Clock INIT 2MHZ
  DrvCLOCK_EnableHighOSC(E_INTERNAL,10);       //Select HSRC
  DrvCLOCK_SelectIHOSC(0);                     //Select internal 2MHZ
  DrvCLOCK_SelectMCUClock(0,0);                //CPU CLOCK IS 'hs_ck/1'
  DrvCLOCK_CalibrateHAO(0);					   //Calibration 2.000MHz
  #endif
  #if defined(HAO_4MHZ)
  //Clock INIT 4MHZ
  DrvCLOCK_EnableHighOSC(E_INTERNAL,10);       //Select HSRC
  DrvCLOCK_SelectIHOSC(1);                     //Select internal 4MHZ
  DrvCLOCK_SelectMCUClock(0,0);                //CPU CLOCK IS 'hs_ck/1'
  DrvCLOCK_CalibrateHAO(1);					   //Calibration 4.000MHz
  #endif
  #if defined(HAO_10MHZ)
  //Clock INIT 10MHZ
  DrvCLOCK_EnableHighOSC(E_INTERNAL,10);       //Select HSRC
  DrvCLOCK_SelectIHOSC(2);                     //Select internal 10MHZ
  DrvCLOCK_SelectMCUClock(0,0);                //CPU CLOCK IS 'hs_ck/1'
  DrvCLOCK_CalibrateHAO(2);					   //Calibration 10.000MHz
  #endif
#endif

#if defined(HSXT)
  DrvCLOCK_EnableHighOSC (E_EXTERNAL,50);
#endif
}

/*----------------------------------------------------------------------------*/
/* Function Name: InitalUart()                                                */
/* Description  : UART Initial Subroutines.                                   */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void InitalUart(void)
{

#if defined(HSRC)
  DrvUART_ClkEnable(1,0);            //Choose the internal HAO as clock source
  #if defined(HAO_2MHZ)
  DrvUART_Open(2000,B9600,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,5);
  #endif
  #if defined(HAO_4MHZ)
  DrvUART_Open(4000,B9600,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,5);
  #endif
  #if defined(HAO_10MHZ)
  DrvUART_Open(10000,B9600,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,5);
  #endif
#endif

#if defined(HSXT)
  DrvUART_ClkEnable(0,0);            //Choose the external OSC as clock source
  DrvUART_Open(4000,B9600,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,5);
#endif

  //2000 : oscillator frequency 2MHz Unit After Calibration HAO = 2000kHz
  //4000 : oscillator frequency 4MHz Unit After Calibration HAO = 4000kHz
  //10000 : oscillator frequency 10MHz Unit After Calibration HAO = 10000kHz
  //None parity
  //8 data bits.
  //5 : Port 2.2 =TX, Port 2.3 =RX

  DrvGPIO_Open(UART_PORT,UART_TXD,E_IO_OUTPUT);
  DrvGPIO_Open(UART_PORT,UART_RXD,E_IO_INPUT);
  DrvGPIO_Open(UART_PORT,UART_RXD|UART_TXD,E_IO_PullHigh);

  DrvGPIO_ClkGenerator(E_HS_CK,1);
  DrvUART_EnableInt(ENABLE,ENABLE);  //Enable UART Tx Interrupt, Enable UART Rx Interrupt
  DrvUART_Close();
  DrvUART_Enable();
}
/*---------------------------------------------------------------------------*/
/* End Of File                                                               */
/*---------------------------------------------------------------------------*/
